#include <qkeycode.h>
#include <qfile.h>
#include <kconfig.h>
#include <kmenubar.h>
#include <kapp.h>
#include <kstddirs.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <dialsetparser.h>
#include <phonelist.h>
#include <dialseteditmain.h>
#include <kradview.h>
#include <krad.h>

#ifdef KeyPress
#undef KeyPress
#endif

KRad::KRad()
{
	setPlainCaption( "K Rapid Acoustic Dialer" );

	dsedit = 0;
	qdial = 0;

	view = new KRadView( this );
	setView( view );

	QPopupMenu *popup = new QPopupMenu;
	popup->insertItem( "&Load Dialset...", this, SLOT( loadDialSet() ) );
	popup->insertSeparator();
	popup->insertItem( "&QuickDial...", this, SLOT( startQuickDial() ) );
	popup->insertItem( "&Dialset Editor...", this, SLOT( startEditor() ) );
	popup->insertSeparator();
	popup->insertItem( "&Quit", kapp, SLOT( quit() ) );
	menuBar()->insertItem( "&File", popup );

	popup = new QPopupMenu;
	popup->insertItem( "DTMF", this, SLOT( dialSetDtmf() ) );
	popup->insertItem( "MFC", this, SLOT( dialSetMfc() ) );
	popup->insertItem( "R2 Forward", this, SLOT( dialSetR2f() ) );
	popup->insertItem( "R2 Backward", this, SLOT( dialSetR2b() ) );

	menuBar()->insertItem( "&Dialset", popup );

	popup = new QPopupMenu;
	popup->insertItem( "Load Phonebook...", this, SLOT( loadPhoneBook() ) );
	popup->insertItem( "Save Phonebook...", this, SLOT( loadPhoneBook() ) );

	menuBar()->insertItem( "&Phonebook", popup );

	menuBar()->insertSeparator();
	menuBar()->insertItem( "&Help", helpMenu( i18n( "krad --- K Rapid Acoustic Dialer\n\n"
			"(c) 1999 Billy Biggs\n" ) ) );

	QString defdialset( locate( "appdata", "dialsets/default.xml" ) );

	QFile deffile( defdialset );
	// Make sure the file exists
	if( !deffile.exists() ) {
		KMessageBox::sorry( this,
			QString( "Default dialset not found!\n\nUsing a blank dialset!" ),
			"KRad: Defaults Not Found" );
		return;
	}
		
	loadDialSet( defdialset );

	resize( 400, 180 );
}

KRad::~KRad()
{
	if( dsedit != 0 ) {
		delete dsedit;
	}

	if( qdial != 0 ) {
		delete qdial;
	}
}

// Session management: save data
void KRad::saveProperties( KConfig * )
{
	// config->writeEntry( "testagent", inputline->text() );
}

// Session management: read data again
void KRad::readProperties( KConfig * )
{
	// inputline->setText( config->readEntry( "testagent" ) );
}

void KRad::loadDialSet()
{
	QString filename( KFileDialog::getOpenFileName( 0, "*.xml" ) );

	if( filename.isNull() ) return;

	loadDialSet( filename );
}

void KRad::loadDialSet( const QString &filename )
{
	// QFile objects are kinda fun
	QFile loadfile( filename );

	// Make sure the file exists
	if( !loadfile.exists() ) {
		KMessageBox::sorry( this,
			QString( "File not found:\n%1" ).arg( filename ),
			"KRad: File Not Found" );
		return;
	}

	// Attempt to open the file
	if( !loadfile.open( IO_ReadOnly ) ) {
		KMessageBox::sorry( this,
			QString( "Can't open file for reading:\n%1" ).arg( filename ),
			"KRad: Can't Open File" );
		return;
	}

	// Read in the entire file (heeheehee)
	QString loadfilexml( loadfile.readAll() );

	DialSetParser kradparser( loadfilexml );

	// Hopefully the parser will set this appropriately
	if( !kradparser.isValid() ) {
		KMessageBox::sorry( this,
			QString( "Invalid KRad dialset: XML Parser Error" ),
			"KRad: Open File Error" );
		return;
	}

	// Grab a dialset if there is one
	if( kradparser.count() > 0 ) {
		view->setDialSet( kradparser.getDialSet( 0 ) );
		if( dsedit != 0 ) {
			dsedit->setDialSet( view->getDialSet() );
		}
	} else {
		KMessageBox::sorry( this,
			QString( "No valid Dialset found in:\n%1" ).arg( filename ),
			"KRad: Open File Error" );
		return;
	}
}

void KRad::dialSetDtmf()
{
	loadDialSet( locate( "appdata", "dialsets/dtmf.xml" ) );
}

void KRad::dialSetMfc()
{
	loadDialSet( locate( "appdata", "dialsets/mfc.xml" ) );
}

void KRad::dialSetR2f()
{
	loadDialSet( locate( "appdata", "dialsets/r2forward.xml" ) );
}

void KRad::dialSetR2b()
{
	loadDialSet( locate( "appdata", "dialsets/r2backward.xml" ) );
}

void KRad::startEditor()
{
	if( dsedit == 0 ) {
		dsedit = new DialSetEditMain;
		dsedit->setDialSet( view->getDialSet() );
	}
	dsedit->show();
}

void KRad::startQuickDial()
{
	if( qdial == 0 ) {
		qdial = new PhoneList;
		connect( qdial, SIGNAL( pleaseDial( QString ) ), view, SLOT( dialNumber( QString ) ) );
		connect( view, SIGNAL( addToList( QString ) ), qdial, SLOT( addToList( QString ) ) );
	}
	qdial->show();
}

void KRad::loadPhoneBook()
{
}

void KRad::savePhoneBook()
{
}

