#include <stdio.h>
#include <qtimer.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qfile.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <kfiledialog.h>
#include <dspoutoss.h>
#include <dspcontrol.h>
#include <dspaction.h>
#include <dialpad.h>
#include <dialset.h>
#include <bhistoryline.h>
#include <dialsetparser.h>
#include <kradview.h>

KRadView::KRadView( QWidget *parent, const char *name )
	: QWidget( parent, name )
{
	dspdev = new DspOutOss( "/dev/dsp" );
	dspdev->setSampleRate( 22050 );
	dspdev->setNumChannels( 1 );
	if( !dspdev->openDevice() ) {
		printf( "Something messed up: %s\n", dspdev->getLastError().ascii() );
		// exit( 1 );
	}
	controller = new DspControl( dspdev );

	QTimer *timer = new QTimer( this );
	connect( timer, SIGNAL( timeout() ), this, SLOT( doTick() ) );
	timer->start( 1 );

	QGridLayout *grid = new QGridLayout( this, 4, 1 );

	grid->setMargin( 2 );
	grid->setSpacing( 5 );
	grid->setResizeMode( QLayout::FreeResize );

	dialpad = new DialPad( 0, this );
	connect( dialpad, SIGNAL( playSound( DspAction * ) ), this, SLOT( doAction( DspAction * ) ) );
	connect( dialpad, SIGNAL( keyPressed( long ) ), this, SLOT( padKeyPressed( long ) ) );
	grid->addMultiCellWidget( dialpad, 0, 3, 0, 0 );

	playline = new BHistoryLine( this );
	connect( playline, SIGNAL( returnPressed() ), this, SLOT( inputReturnPressed() ) );
	grid->addWidget( playline, 4, 0 );

	setname = new QLabel( this );
	setname->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	setname->setText( "KRad" );
	setname->setFixedHeight( setname->sizeHint().height() );
	grid->addWidget( setname, 0, 1 );

	rec = new QCheckBox( "Rec", this );
	rec->setChecked( false );
	grid->addWidget( rec, 1, 1 );

	QCheckBox *mute = new QCheckBox( "Mute", this );
	mute->setChecked( false );
	grid->addWidget( mute, 2, 1 );

	QPushButton *showcodes = new QPushButton( "Show Codes", this );
	connect( showcodes, SIGNAL( pressed() ), this, SLOT( showKeysPressed() ) );
	connect( showcodes, SIGNAL( released() ), this, SLOT( showKeysReleased() ) );
	grid->addWidget( showcodes, 3, 1 );

	QPushButton *addtolist = new QPushButton( "Add to QuickDial", this );
	connect( addtolist, SIGNAL( clicked() ), this, SLOT( addToListClicked() ) );
	grid->addWidget( addtolist, 4, 1 );
}

KRadView::~KRadView()
{
	delete controller;
	delete dspdev;
}

void KRadView::doAction( DspAction *curaction )
{
	if( curaction != 0 ) {
		controller->queueAction( curaction );
		controller->queueAction( new DspSilenceAction( 50 ) );
	}
}

void KRadView::doTick()
{
	controller->doTick();
}

void KRadView::inputReturnPressed()
{
	QString text = playline->text();
	playline->clear();

	for( unsigned int i = 0; i < text.length(); i++ ) {
		if( text[ i ] == '1' ) { dialpad->hitKey( 1 ); }
		if( text[ i ] == '2' ) { dialpad->hitKey( 2 ); }
		if( text[ i ] == '3' ) { dialpad->hitKey( 3 ); }
		if( text[ i ] == '4' ) { dialpad->hitKey( 4 ); }
		if( text[ i ] == '5' ) { dialpad->hitKey( 5 ); }
		if( text[ i ] == '6' ) { dialpad->hitKey( 6 ); }
		if( text[ i ] == '7' ) { dialpad->hitKey( 7 ); }
		if( text[ i ] == '8' ) { dialpad->hitKey( 8 ); }
		if( text[ i ] == '9' ) { dialpad->hitKey( 9 ); }
		if( text[ i ] == '*' ) { dialpad->hitKey( 10 ); }
		if( text[ i ] == '0' ) { dialpad->hitKey( 11 ); }
		if( text[ i ] == '#' ) { dialpad->hitKey( 12 ); }

		if( text[ i ].lower() == 'a' ) { dialpad->hitKey( 13 ); }
		if( text[ i ].lower() == 'b' ) { dialpad->hitKey( 14 ); }
		if( text[ i ].lower() == 'c' ) { dialpad->hitKey( 15 ); }
		if( text[ i ].lower() == 'd' ) { dialpad->hitKey( 16 ); }

		if( text[ i ].lower() == 'e' ) { dialpad->hitKey( 17 ); }
		if( text[ i ].lower() == 'f' ) { dialpad->hitKey( 18 ); }
		if( text[ i ].lower() == 'g' ) { dialpad->hitKey( 19 ); }
		if( text[ i ].lower() == 'h' ) { dialpad->hitKey( 20 ); }

		if( text[ i ].lower() == 'i' ) { dialpad->hitKey( 21 ); }
		if( text[ i ].lower() == 'j' ) { dialpad->hitKey( 22 ); }
		if( text[ i ].lower() == 'k' ) { dialpad->hitKey( 23 ); }
		if( text[ i ].lower() == 'l' ) { dialpad->hitKey( 24 ); }

		if( text[ i ].lower() == ',' ) { doAction( new DspSilenceAction( 60 ) ); }
	}
}

void KRadView::padKeyPressed( long id )
{
	if( rec->isChecked() ) {
		playline->insert( DialPad::getKeyCode( id ) );
	}
}

void KRadView::setDialSet( DialSet *newset )
{
	dialpad->setDialSet( newset );
	setname->setText( newset->getName() );
}

void KRadView::showKeysPressed()
{
	dialpad->showKeyCodes( true );
}

void KRadView::showKeysReleased()
{
	dialpad->showKeyCodes( false );
}

DialSet *KRadView::getDialSet()
{
	return dialpad->getDialSet();
}

void KRadView::dialNumber( QString number )
{
	playline->setText( number );
	inputReturnPressed();
}

void KRadView::addToListClicked()
{
	addToList( playline->text() );
}

